@@ -8,11 +8,13 @@ Importing this module makes "bdist_esky" available as a distutils command.
 This command will freeze the given scripts and package them into a zipfile
 named with the application name, version and platform.
 
-The resulting zipfile is conveniently in the format expected by the default
-SimpleVersionFinder.  It will be named "appname-version.platform.zip"
+The resulting zipfile is conveniently in the format expected by the class
+DefaultVersionFinder.  It will be named "appname-version.platform.zip"
 
 """
 
+from __future__ import with_statement
+
 
 import os
 import re
@@ -31,6 +33,15 @@ if sys.platform == "win32":
     from xml.dom import minidom
 
 
+#  setuptools likes to be imported before anything else that
+#  might monkey-patch distutils.  We don't actually use it,
+#  this is just to avoid errors.
+try:
+    import setuptools
+except ImportError:
+    pass
+
+
 _FREEZERS = {}
 try:
     from esky.bdist_esky import f_bbfreeze
@@ -42,6 +53,57 @@ try:
     _FREEZERS["py2exe"] = f_py2exe
 except ImportError:
     _FREEZERS["py2exe"] = None
+try:
+    from esky.bdist_esky import f_cxfreeze
+    _FREEZERS["cxfreeze"] = f_cxfreeze
+    _FREEZERS["cx_Freeze"] = f_cxfreeze
+    _FREEZERS["cx_freeze"] = f_cxfreeze
+except ImportError:
+    _FREEZERS["cxfreeze"] = None
+    _FREEZERS["cx_Freeze"] = None
+    _FREEZERS["cx_freeze"] = None
+
+
+class Executable(str):
+    """Class to hold information about a specific executable.
+
+    This class provides a uniform way to specify extra meta-data about
+    a frozen executable.  By setting various keyword arguments, you can
+    specify e.g. the icon, and whether it is a gui-only script.
+
+    This is a subclass of str() so that it can appear directly in the
+    "scripts" argument of the setup() function; I know it's ugly, but
+    it works.
+    """
+
+    def __new__(cls,script,**kwds):
+        return str.__new__(cls,script)
+
+    def __init__(self,script,icon=None,gui_only=None,**kwds):
+        str.__init__(script)
+        self.script = script
+        self.icon = icon
+        self._gui_only = gui_only
+        self._kwds = kwds
+
+    @property
+    def name(self):
+        nm = os.path.basename(self.script)
+        if nm.endswith(".py"):
+            nm = nm[:-3]
+        elif nm.endswith(".pyw"):
+            nm = nm[:-4]
+        if sys.platform == "win32":
+            nm += ".exe"
+        return nm
+
+    @property
+    def gui_only(self):
+        if self._gui_only is None:
+            return self.script.endswith(".pyw")
+        else:
+            return self._gui_only
+
 
 
 class bdist_esky(Command):
@@ -93,7 +155,7 @@ class bdist_esky(Command):
                     ('bootstrap-module=', None,
                      "module to use for bootstrapping the application"),
                     ('bundle-msvcrt=', None,
-                     "whether to bundle MSVCR as private assembly"),
+                     "whether to bundle MSVCRT as private assembly"),
                     ('includes=', None,
                      "list of modules to specifically include"),
                     ('excludes=', None,
@@ -114,9 +176,11 @@ class bdist_esky(Command):
     def finalize_options(self):
         self.set_undefined_options('bdist',('dist_dir', 'dist_dir'))
         if self.freezer_module is None:
-            try:
-                freezer = _FREEZERS.itervalues().next()
-            except StopIteration:
+            for freezer_module in ("py2exe","bbfreeze","cxfreeze"):
+                self.freezer_module = _FREEZERS[freezer_module]
+                if self.freezer_module is not None:
+                    break
+            else:
                 err = "no supported freezer modules found"
                 err += " (try installing bbfreeze)"
                 raise RuntimeError(err)
@@ -159,11 +223,14 @@ class bdist_esky(Command):
         zf.close()
         shutil.rmtree(self.bootstrap_dir)
 
-    def get_scripts(self):
-        """Yield paths of all scripts to be included in the freeze."""
+    def get_executables(self):
+        """Yield an Executable instance for each script to be frozen."""
         if self.distribution.has_scripts():
             for s in self.distribution.scripts:
-                yield s
+                if isinstance(s,Executable):
+                    yield s
+                else:
+                    yield Executable(s)
 
     def get_data_files(self):
         """Yield (source,destination) tuples for data files.
