@@ -5,8 +5,8 @@
   esky.finder:  VersionFinder implementations for esky
 
 This module provides the default VersionFinder implementations for esky. The
-abstract base class 'VersionFinder' defines the expected interface, while 
-'SimpleVersionFinder' provides a simple default implementation that hits a
+abstract base class "VersionFinder" defines the expected interface, while 
+"DefaultVersionFinder" provides a simple default implementation that hits a
 specified URL to look for new versions.
 
 """
@@ -28,108 +28,101 @@ class VersionFinder(object):
     """Base VersionFinder class.
 
     This class defines the interface expected of a VersionFinder object.
-    It will be given three properties at initialisation time:
-
-        appname:   name of the application being managed
-        platform:  platform for which the application was produced
-        workdir:   directory in which updates can be downloaded, extracted, etc
-
     The important methods expected from any VersionFinder are:
 
         cleanup:  perform maintenance/cleanup tasks in the workdir
                   (e.g. removing old or broken downloads)
 
-        find_versions:  get a list of all available versions
+        find_versions:  get a list of all available versions for a given esky
 
         fetch_version:  make the specified version available locally
                         (e.g. download it from the internet)
 
         has_version:  check that the specified version is available locally
 
-        prepare_version:  extract the specific version into a directory
-                          that can be linked into the application
-
     """
 
-    def __init__(self,appname=None,platform=None,workdir=None):
-        self.appname = appname
-        self.platform = platform
-        self.workdir = workdir
+    def __init__(self):
+        pass
 
-    def cleanup(self):
+    def cleanup(self,app):
         """Perform maintenance tasks in the working directory."""
         pass
 
-    def find_versions(self):
+    def find_versions(self,app):
         """Find available versions of the app, returned as a list."""
         raise NotImplementedError
 
-    def fetch_version(self,version):
-        """Fetch a specific version of the app into local storage."""
+    def fetch_version(self,app,version):
+        """Fetch a specific version of the app into a local directory."""
         raise NotImplementedError
 
-    def has_version(self,version):
+    def has_version(self,app,version):
         """Check whether a specific version of the app is available locally."""
         raise NotImplementedError
 
-    def prepare_version(self,version):
-        """Extract a specific version of the app into a local directory."""
-        raise NotImplementedError
 
 
-class SimpleVersionFinder(VersionFinder):
-    """VersionFinder implementing simple download scheme.
+class DefaultVersionFinder(VersionFinder):
+    """VersionFinder implementing simple default download scheme.
 
-    SimpleVersionFinder expects to be given a download url, which it will
+    DefaultVersionFinder expects to be given a download url, which it will
     hit looking for new versions packaged as zipfiles.  These are simply
     downloaded and extracted on request.
 
     Zipfiles suitable for use with this class can be produced using the
     "bdist_esky" distutils command.
+
+    This class will eventually grow support for applying differential updates,
+    but I haven't implemented it yet...
     """
 
-    def __init__(self,download_url,**kwds):
+    def __init__(self,download_url):
         self.download_url = download_url
-        super(SimpleVersionFinder,self).__init__(**kwds)
+        super(DefaultVersionFinder,self).__init__()
         self.version_urls = {}
 
-    def _ensure_work_dir(self,nm):
+    def _workdir(self,app,nm):
+        """Get full path of named working directory, inside the given app."""
+        workdir = os.path.join(app._get_update_dir(),nm)
         try:
-            os.makedirs(os.path.join(self.workdir,nm))
+            os.makedirs(workdir)
         except OSError, e:
             if e.errno not in (17,183):
                 raise
+        return workdir
 
-    def cleanup(self):
-        self._ensure_work_dir("downloads")
-        dldir = os.path.join(self.workdir,"downloads")
+    def cleanup(self,app):
+        dldir = self._workdir(app,"downloads")
         for nm in os.listdir(dldir):
             os.unlink(os.path.join(dldir,nm))
-        self._ensure_work_dir("unpack")
-        updir = os.path.join(self.workdir,"unpack")
+        updir = self._workdir(app,"unpack")
         for nm in os.listdir(updir):
-            os.unlink(os.path.join(updir,nm))
+            shutil.rmtree(os.path.join(updir,nm))
+        rddir = self._workdir(app,"ready")
+        for nm in os.listdir(rddir):
+            shutil.rmtree(os.path.join(rddir,nm))
 
     def open_url(self,url):
         return urllib2.urlopen(url)
 
-    def find_versions(self):
-        downloads = self.open_url(self.download_url).read()
-        version_re = "(?P<version>[a-zA-Z0-9\\.-]+)"
-        version_re = join_app_version(self.appname,version_re,self.platform)
+    def find_versions(self,app):
+        # TODO: would be nice not to have to guess encoding here.
+        downloads = self.open_url(self.download_url).read().decode("utf-8")
+        version_re = "(?P<version>[a-zA-Z0-9\\.-_]+)"
+        version_re = join_app_version(app.name,version_re,app.platform)
         link_re = "href=['\"](?P<href>(.*/)?%s.zip)['\"]" % (version_re,)
         for match in re.finditer(link_re,downloads):
             self.version_urls[match.group("version")] = match.group("href")
         return self.version_urls.keys()
 
-    def fetch_version(self,version):
-        self._ensure_work_dir("downloads")
+    def fetch_version(self,app,version):
         try:
             url = self.version_urls[version]
         except KeyError:
             raise EskyVersionError(version)
         infile = self.open_url(urljoin(self.download_url,url))
-        outfilenm = self._download_name(version)+".part"
+        outfilenm = self._download_name(app,version)+".part"
         outfile = open(outfilenm,"wb")
         try:
             data = infile.read(1024*512)
@@ -144,21 +137,29 @@ class SimpleVersionFinder(VersionFinder):
         else:
             infile.close()
             outfile.close()
-            os.rename(outfilenm,self._download_name(version))
+            os.rename(outfilenm,self._download_name(app,version))
+        return self._prepare_version(app,version)
 
-    def _download_name(self,version):
-        version = join_app_version(self.appname,version,self.platform)
-        return os.path.join(self.workdir,"downloads","%s.zip" % (version,))
+    def _download_name(self,app,version):
+        version = join_app_version(app.name,version,app.platform)
+        return os.path.join(self._workdir(app,"downloads"),"%s.zip"%(version,))
 
-    def has_version(self,version):
-        return os.path.exists(self._download_name(version))
+    def _ready_name(self,app,version):
+        version = join_app_version(app.name,version,app.platform)
+        return os.path.join(self._workdir(app,"ready"),version)
 
-    def prepare_version(self,version):
-        self._ensure_work_dir("unpack")
-        dlpath = self._download_name(version)
-        vdir = join_app_version(self.appname,version,self.platform)
-        uppath = os.path.join(self.workdir,"unpack")
-        #  Anything in the root of the zipfile is part of the boostrap
+    def _prepare_version(self,app,version):
+        """Prepare the request version from downloaded data.
+
+        This method is responsible for unzipping the downloaded version
+        and making it available as a local directory.  When I implement
+        differential updates it will also be responsible for applying them.
+        """
+        uppath = self._workdir(app,"unpack")
+        dlpath = self._download_name(app,version)
+        rdpath = self._ready_name(app,version)
+        vdir = join_app_version(app.name,version,app.platform)
+        #  Anything in the root of the zipfile is part of the bootstrap
         #  env, so it gets placed in a special directory.
         def name_filter(nm):
             if not nm.startswith(vdir):
@@ -168,6 +169,12 @@ class SimpleVersionFinder(VersionFinder):
         bspath = os.path.join(uppath,vdir,"esky-bootstrap")
         if not os.path.isdir(bspath):
             os.makedirs(bspath)
-        return os.path.join(uppath,vdir)
+        os.rename(os.path.join(uppath,vdir),rdpath)
+        os.unlink(dlpath)
+        return rdpath
+
+    def has_version(self,app,version):
+        return os.path.exists(self._ready_name(app,version))
+
 
 
