@@ -17,13 +17,16 @@ eskytester.yes_i_am_working()
 eskytester.yes_my_deps_are_working()
 eskytester.yes_my_data_is_installed()
 
-#  Upgrade to the next version (0.2, even though 0.3 is available)
 assert sys.frozen
 app = esky.Esky(sys.executable,"http://localhost:8000/dist/")
 assert app.name == "eskytester"
 assert app.version == "0.1"
 assert app.find_update() == "0.3"
-app.install_update("0.2")
+assert os.path.isfile(os.path.join(app.appdir,"script1"+dotexe))
+
+#  Upgrade to the next version (0.2, even though 0.3 is available)
+app.install_version("0.2")
+app.reinitialize()
 assert app.name == "eskytester"
 assert app.version == "0.2"
 assert app.find_update() == "0.3"
