@@ -1,4 +1,6 @@
 
+from __future__ import with_statement
+
 import sys
 import os
 import unittest
@@ -28,10 +30,24 @@ try:
     import py2exe
 except ImportError:
     py2exe = None
+try:
+    import cx_Freeze
+except ImportError:
+    cx_Freeze = None
 
 sys.path.append(os.path.dirname(__file__))
 
 
+if not hasattr(HTTPServer,"shutdown"):
+    import socket
+    def socketserver_shutdown(self):
+        try:
+            self.socket.close()
+        except socket.error:
+            pass
+    HTTPServer.shutdown = socketserver_shutdown
+
+
 class TestEsky(unittest.TestCase):
 
   if bbfreeze is not None:
@@ -44,6 +60,11 @@ class TestEsky(unittest.TestCase):
         """Build and launch a self-testing esky application using py2exe."""
         self._run_eskytester({"bdist_esky":{"freezer_module":"py2exe"}})
 
+  if cx_Freeze is not None:
+    def test_esky_cxfreeze(self):
+        """Build and launch a self-testing esky application using cx_Freeze."""
+        self._run_eskytester({"bdist_esky":{"freezer_module":"cxfreeze"}})
+
   def _run_eskytester(self,options):
     """Build and run the eskytester app using the given distutils options.
 
@@ -51,6 +72,7 @@ class TestEsky(unittest.TestCase):
     sequence of tests performed range across "script1.py" to "script3.py".
     """
     olddir = os.path.abspath(os.curdir)
+    server = None
     try:
         platform = get_platform()
         deploydir = "deploy.%s" % (platform,)
@@ -81,13 +103,15 @@ class TestEsky(unittest.TestCase):
         #  Serve the updates at http://localhost:8000/dist/
         print "running local update server"
         server = HTTPServer(("localhost",8000),SimpleHTTPRequestHandler)
-        threading.Thread(target=server.serve_forever).start()
+        server_thread = threading.Thread(target=server.serve_forever)
+        server_thread.daemon = True
+        server_thread.start()
         #  Set up the deployed esky environment for the initial version
         zfname = os.path.join("dist","eskytester-0.1.%s.zip"%(platform,))
         os.mkdir(deploydir)
         extract_zipfile(zfname,deploydir)
-        #  Run the first script, which will perform the necessary tests
-        #  and write "tests-completed" file when done.
+        #  Run the first script, which will perform the necessary tests,
+        #  launch script2 and script3, and write the file "tests-completed".
         if os.path.exists("tests-completed"):
             os.unlink("tests-completed")
         if sys.platform == "win32":
@@ -97,12 +121,14 @@ class TestEsky(unittest.TestCase):
         print "spawning eskytester application"
         p = subprocess.Popen(cmd,stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
         (stdout,_) = p.communicate()
-        sys.stdout.write(stdout)
+        sys.stdout.write(stdout.decode())
         assert p.returncode == 0
         assert os.path.exists("tests-completed")
         os.unlink("tests-completed")
     finally:
         os.chdir(olddir)
+        if server:
+            server.shutdown()
 
  
   def test_esky_locking(self):
@@ -203,16 +229,17 @@ class TestEsky(unittest.TestCase):
     readme = os.path.join(dirname(dirname(dirname(__file__))),"README.txt")
     if not os.path.isfile(readme):
         f = open(readme,"wb")
-        f.write(esky.__doc__)
+        f.write(esky.__doc__.encode())
         f.close()
     else:
         f = open(readme,"rb")
         if f.read() != esky.__doc__:
             f.close()
             f = open(readme,"wb")
-            f.write(esky.__doc__)
+            f.write(esky.__doc__.encode())
             f.close()
 
+
 class TestFSTransact(unittest.TestCase):
     """Testcases for FSTransact."""
 
@@ -229,11 +256,11 @@ class TestFSTransact(unittest.TestCase):
         if not os.path.isdir(os.path.dirname(self.path(path))):
             os.makedirs(os.path.dirname(self.path(path)))
         with open(self.path(path),"wb") as f:
-            f.write(contents)
+            f.write(contents.encode())
 
     def assertContents(self,path,contents):
         with open(self.path(path),"rb") as f:
-            self.assertEquals(f.read(),contents)
+            self.assertEquals(f.read().decode(),contents)
 
     def test_move_file(self):
         self.setContents("file1","hello world")
