@@ -10,11 +10,16 @@ import os
 import re
 import shutil
 import zipfile
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 from distutils.util import get_platform as _distutils_get_platform
 
-from esky.bootstrap import get_best_version, split_app_version, \
-                           join_app_version, parse_version
+from esky.bootstrap import get_best_version, get_all_versions, \
+                           is_version_dir, is_installed_version_dir, \
+                           split_app_version, join_app_version, parse_version
 
 
 def extract_zipfile(source,target,name_filter=None):
@@ -26,14 +31,21 @@ def extract_zipfile(source,target,name_filter=None):
     in the target directory.
     """
     zf = zipfile.ZipFile(source,"r")
+    if hasattr(zf,"open"):
+        zf_open = zf.open
+    else:
+        def zf_open(nm,mode):
+            return StringIO(zf.read(nm))
     for nm in zf.namelist():
+        if nm.endswith("/"):
+            continue
         if name_filter:
             outfilenm = os.path.join(target,name_filter(nm))
         else:
             outfilenm = os.path.join(target,nm)
         if not os.path.isdir(os.path.dirname(outfilenm)):
             os.makedirs(os.path.dirname(outfilenm))
-        infile = zf.open(nm,"r")
+        infile = zf_open(nm,"r")
         try:
             outfile = open(outfilenm,"wb")
             try:
